#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 10:49
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : pmid_parser.py
from __future__ import annotations
from typing import Union, List, Optional, Dict, Text
from collections import Iterable
import os, json, re
import logging

from tqdm import tqdm

from .base_parser import BasePaeser
from .schemas.schema import Example, Entity
from liyi_cute.utils.common import iter_file_groups
logger = logging.getLogger(__file__)

class PmidParser(BasePaeser):
    exts = {".json"}
    support_save = {".pkl", ".json" }
    support_task_name = {"ner"} #  "ner", "rel" "env", "attr"
    types = {"T"}
    def __init__(self, task_name, ignore_types: Optional[Iterable[str]] = None, error: str = "raise"):
        super().__init__(task_name,ignore_types, error)

    def _parse_json(self, input_path:str, encoding:str, key)->Example:
        entities, relations, events = [], [], []

        js_data = json.load(open(input_path, 'r', encoding="utf-8"))
        text = ""
        annotations = []
        for item in js_data["passages"]:
            if item["text"]:
                text = text + item["text"] + " "
                for anns in item["annotations"]:
                    if anns["text"].strip() != "":
                        annotations.append([anns["locations"][0]["offset"],
                                            anns["locations"][0]["offset"] + anns["locations"][0]["length"],
                                            anns["text"],
                                            anns["infons"]["type"]])
        text = text.rstrip()

        ## check annotations
        annotations = self.check_annotations(text, annotations)

        # Format entities.
        entities = self._format_entities(annotations)
        self._check_entities(entities.values())

        example = Example(text=text,
                          task_name=self.task_name,
                          entities=list(entities.values()),
                          relations=relations,
                          events=events,
                          id=key)
        return example

    def _check_entities(self, entities):  # pylint: disable=no-self-use
        pool = {}
        for entity in entities:
            id_ = pool.setdefault((entity.start, entity.end), entity.id)
            if id_ != entity.id:
                self._raise(
                    RuntimeError(
                        "Detected identical span for"
                        f" different entities: [{id_}, {entity.id}]"
                    )
                )

    def _format_entities(
            self, annotations
    )->Dict:
        entities = {}
        num_entities = 0
        for ann in annotations:
            entities.update({
                "T" + str(num_entities+1):Entity(
                    mention=ann[2],
                    type=ann[3],
                    start=ann[0],
                    end=ann[1],
                    id = "T" + str(num_entities+1)
                )
            })
            num_entities = num_entities+1
        return entities


    @staticmethod
    def check_annotations(text: str, annotations: List) -> List:
        """
        :param text:
        :param annotations:
        :return:
        """
        for an in annotations:
            start_id = an[0]
            end_ids = an[1]
            if text[start_id:end_ids] != an[2]:
                logger.warning("text: " + text)
                logger.warning("start_ids: " + str(start_id))
                logger.warning("end_ids: " + str(end_ids))
                logger.warning("text label: " + text[start_id:end_ids])
                logger.warning("annotations content: " + an[2])
                an[2] = text[start_id:end_ids]

        return annotations

    def parse(self,  dirname: Union[str, bytes, os.PathLike], encoding: str = "utf-8")->List[Example]:
        examples = []
        file_groups = iter_file_groups(
            dirname,
            self.exts,
            missing="error" if self.error == "raise" else "ignore",
        )
        for key, json_file in tqdm(file_groups):
            example = self._parse_json(json_file[0], encoding=encoding, key=key)
            examples += [example]

        examples.sort(key=lambda x: x.id if x.id is not None else "")
        self.examples = examples

        return self.examples







#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 18:25
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : extract_tag.py

from spacy.gold import biluo_tags_from_offsets
import spacy
nlp = spacy.load("en_core_web_lg")
import json
from spacy import tokenizer
data_path = "../../datasets/output/test.json"
data = json.loads(open(data_path, "r", encoding='utf-8').read())

for content in data:
    for k in content:
        text = content["text"]
        entities = content["entities"]
        entities = [(en["start"],en["end"],en["type"]) for en in entities]
        doc = nlp(text)
        tags = biluo_tags_from_offsets(doc, entities)
        tokens = [token.text for token in doc]

        assert len(tags) == len(tokens), "not equel"


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/30 19:40
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : test.py

import pickle
import spacy
from liyi_cute.shared.imports.pmid_parser import PmidParser
from spacy.gold import docs_to_json, biluo_tags_from_offsets, spans_from_biluo_tags
nlp = spacy.load('en_core_web_sm')

# obj = PmidParser(task_name="ner", error="ignore")
# examples = obj.parse(r"../data/pmcid")
#
# with open("../data/output/pmcid.pkl", "wb") as f:
#     pickle.dump(examples, f)

with open("../data/output/pmid.pkl","rb") as f:
    examples=pickle.load(f)

print(len(examples))
## 转cnoll格式的文件
"""
[{"id":"001",
"text":""
"p":[{
      "id":"1",
      "short_text":""
      "text_list":[],
      "tag_list":[],
      "entities":[],
      "relations":[]
}]
}
]

"""
TRAIN_DATA = []
for example in examples:
    text = example.text
    id = example.id
    entities_list = example.entities
    entities = []
    TRAIN_DATA.append((id, text, {"entities": [(ent.start, ent.end, ent.type.upper())for ent in entities_list]}))

all_data = []
all_tags = []
docs = []
for id, text, annot in TRAIN_DATA:
    doc = nlp(text)
    try:
        tags = biluo_tags_from_offsets(doc, annot['entities'])
    except Exception as e:
        print(id, "---")
        continue
    entities = spans_from_biluo_tags(doc, tags)
    doc.ents = entities
    data = []
    tags = []
    for token in doc:
        if token.is_sent_end:
            ent_iob = token.ent_iob_
            if token.ent_iob_ !="O":
                ent_iob = token.ent_iob_ + "-" + token.ent_type_
            data.append(token.text)
            tags.append(ent_iob)
            all_data.append(data)
            all_tags.append(tags)
            data = []
            tags = []
        else:
            data.append(token.text)
            ent_iob = token.ent_iob_
            if token.ent_iob_ != "O":
                ent_iob = token.ent_iob_ + "-" + token.ent_type_
            tags.append(ent_iob)


# a = docs_to_json(docs)
txt_w = open("../data/output/pmid.txt", 'w', encoding='utf-8')
for idx in range(len(all_data)):
    assert len(all_data[idx])==len(all_tags[idx]), "长度不一致"
    cont = list(zip(all_data[idx],all_tags[idx])) #[(),()]
    for c in cont:
        txt_w.write("\t".join(c)+"\n")
    txt_w.write("\n")

txt_w.close()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/5 16:20
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : version.py.py
from __future__ import annotations

__version__ = '0.0.1'


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 12:31
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : convert.py
from __future__ import annotations

from typing import Text, Any, List

from liyi_cute.shared.imports.schemas.schema import Example
from liyi_cute.utils.io import write_text_file, examples_as_json


def convert_example_data(
    examples: List[Example], out_file: Text, output_format: Text, **kwargs: Any
) -> None:
    """
    :param examples:
    :param out_file:
    :param output_format:
    :return:
    """

    if not (isinstance(examples, List)):
        raise TypeError("data_file type is not in (Example or Text)")

    if output_format == "json":
        output= examples_as_json(examples, **kwargs)
    else:
        raise ValueError("....")

    write_text_file(file_path=out_file, content=output)










#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/4 16:41
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : test2.py
import os

file_name = "test.txt"
root_path = "../data/conll/"

all_data = []
data = []
all_tag = []
tag = []
with open(os.path.join(root_path, file_name), 'r', encoding="utf-8") as fr:
    lines = fr.readlines()
    for line in lines:
        line = line.strip()
        if line:
            line_list = line.split(" ")
            if len(line_list)==4:
                data.append(line_list[0])
                tag.append(line_list[3])
        else:
            all_data.append(data)
            all_tag.append(tag)
            data = []
            tag = []

with open(os.path.join(root_path, "test.txt"), "w", encoding="utf-8") as fw:
    for d, t in list(zip(all_data, all_tag)):
        for k in list(zip(d, t)):
            fw.write("\t".join(k))
            fw.write("\n")
        fw.write("\n")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 10:06
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : postprocess.py
from __future__ import annotations

from typing import Dict, List, Tuple, Union, Text

from tqdm import tqdm

from liyi_cute.shared.imports.schemas.schema import TraningExample, TraningNerExample

"""
输出内容
pre_ner_tag = []

"""

def NerPostprocessing(pre_neg_tags:List[List],
                      train_samples:List[TraningNerExample])-> Tuple[List, List[List], List[List]]:

    id = ""
    test_texts = []
    text_pre_tags = []
    offset_mappings = []

    offset_mapping = []
    text_string = ""
    pre_tag = []
    for index, example in enumerate(train_samples):
        if index==0:
            text = example.text
            id = example.id
            text_string+=example.cut_text
            pre_tag+= pre_neg_tags[index]
            offset_mapping += example.offset_mapping
        elif id==example.id:
            text = example.text
            id=example.id
            text_string += example.cut_text
            pre_tag += pre_neg_tags[index]
            offset_mapping += example.offset_mapping
        else:
            assert len(text)==len(text_string), "句子长度不相等"
            test_texts.append(text_pre_tags)
            offset_mappings.append(offset_mapping)
            text_pre_tags.append(pre_tag)
            ## 不相等的重新初始化数据
            id = example.id
            text = example.text
            text_string = example.cut_text
            pre_tag = pre_neg_tags[index]
            offset_mapping = example.offset_mapping

    if text_string:
        test_texts.append(text_pre_tags)
        offset_mappings.append(offset_mapping)
        text_pre_tags.append(pre_tag)

    return test_texts, offset_mappings, text_pre_tags

def postion_to_string(pre_tags, entitie, offset_mapping:List[Tuple], text, ids)->Dict:
    tag_type = pre_tags[entitie[0]].split("-")[1]
    start = offset_mapping[entitie[0]][0]
    end = offset_mapping[entitie[-1]][1]
    return {"id": "T"+str(ids), "mention":text[start:end], "start":start, "end":end, "type":tag_type}

def entity_to_json_v1(pre_tags, offset_mapping, text):
    entities = []
    entitie = []
    ids = 1
    for index, tag in enumerate(pre_tags):
        if tag[0]=="B" and len(entitie)==0:
            entitie.append(index)
        elif tag[0]=="B" and len(entitie)!=0:
            entities.append(postion_to_string(pre_tags, entitie, offset_mapping, text, ids))
            ids += 1
            entitie = [index]
        elif tag[0]=="I":
            entitie.append(index)
        else:
            if entitie:
                entities.append(postion_to_string(pre_tags, entitie, offset_mapping, text, ids))
                ids += 1
            entitie = []

    return {"text": text, "entities": entities}

def result_to_json_v2(strings, tags):
    """
    :param strings:
    :param tags:
    :return:
    """
    item = {"string":strings, "entities":[]}

    entity_name = ""
    entity_start = 0
    idx = 0

    for word, tag in zip(strings, tags):
        print(word, tag)
        if tag[0] == "S":
            item['entities'].append({"word":word, "start":idx+1, "type":tag[2:]})
        elif tag[0] == "B":
            entity_name = entity_name + word
            entity_start = idx
        elif tag[0] == "I":
            entity_name = entity_name + word
        elif tag[0] == "E":
            entity_name = entity_name + word
            item['entities'].append({"mention":entity_name, "start":entity_start, "end":idx+1, "type":tag[2:]})
        else:
            entity_name=""
            entity_start = idx
        idx = idx + 1
    return item


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/4 22:53
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : test4.py
from liyi_cute.shared.imports.schemas.schema import Entity, Example


def _en_serialization (data, tag, mode):
    text = ""
    entities = []
    tag_len = len(tag)
    index = 0
    ids = 1
    while index < tag_len:
        text += data[index] + " "
        if tag[index][0] == "B":
            type = tag[index].split("-")[1]
            start = len(text) - len(data[index])
            mention = data[index] + " "
            while index + 1 < tag_len and tag[index + 1] == "I":
                index += 1
                text += data[index] + " "
                mention += data[index] + " "
            end = len(text)
            entities.append(Entity(id=f"T{str(ids)}", start=start, end=end, mention=mention.rstrip(), type=type))
            ids += 1
    return Example(id=mode, task_name="ner", text=text, entities=entities)

data = []
tag = []
mode ="train"


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 17:09
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : exceptions.py
from __future__ import annotations

class LiyiException(Exception):
    """Base exception class for all errors."""

class FileIOException(LiyiException):
    """error while doing file IO."""

class FileNotFoundException(LiyiException, FileNotFoundError):
    """file doesn't exist."""

class NotImplementedException(NotImplementedError):
    """error while doing file IO."""

class NotExistException(ValueError):
    """error while doing file IO."""

class ParamNotExistException(LiyiException):
    """miss parameter"""

class NotSupportedException(LiyiException):
    """not support format error"""


# -*- coding:UTF-8 -*-

# author:user
# contact: test@test.com
# datetime:2022/3/29 14:21
# software: PyCharm

"""
文件说明：
    
"""
from marshmallow import Schema, fields, post_load

from .schema import Entity, Relation, Event, Example, TraningNerExample, TraningExample


class EntitySchema(Schema):
    mention = fields.Str()
    type = fields.Str()
    start = fields.Int()
    end = fields.Int()
    id = fields.Str(default=None)

    @post_load
    def make_entity(self, data, **kwargs):
        return Entity(**data)


class RelationSchema(Schema):
    type = fields.Str()
    arg1 = fields.Nested(EntitySchema())
    arg2 = fields.Nested(EntitySchema())
    id = fields.Str(default=None)

    @post_load
    def make_relation(self, data, **kwargs):
        return Relation(**data)


class EventSchema(Schema):
    id = fields.Str(default=None)

    @post_load
    def make_event(self, data, **kwargs):
        return Event(**data)

class ExampleSchema(Schema):
    text = fields.Str()
    entities = fields.List(fields.Nested(EntitySchema()))
    relations = fields.List(fields.Nested(RelationSchema()))
    events = fields.List(fields.Nested(EventSchema()))
    id = fields.Str(default=None)
    task_name = fields.Str(default=None)

    @post_load
    def make_example(self, data, **kwargs):
        return Example(**data)


"""
######################## 处理文本 ######################
"""

class TraningExampleSchema(Schema):
    text = fields.Str()
    cut_text = fields.Str()
    sub_id = fields.Integer()
    cut_number = fields.Integer(default=0)
    entities = fields.List(fields.Nested(EntitySchema()))
    cut_entities = fields.List(fields.Nested(EntitySchema()))
    cut_len = fields.Integer(default=0)
    id = fields.Str(default=None)
    task_name = fields.Str(default=None)
    input_ids = fields.List(fields.Integer())
    attention_mask = fields.List(fields.Integer(), default=list())
    token_type_ids  = fields.List(fields.Integer(), default=list())


class TraningNerExampleSchema(TraningExampleSchema):
    true_ner_tag = fields.List(fields.Str(), default=list())
    pre_ner_tag = fields.List(fields.Str(), default=list())
    offset_mapping = fields.List(fields.Tuple((fields.Integer(), fields.Integer())), default=list())

    @post_load
    def make_train_ner_example(self, data, **kwargs):
        return TraningNerExample(**data)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/1 23:32
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : example_cut_sent.py
from liyi_cute.processor.preprocess import cut_sent
from liyi_cute.shared.imports.pmid_parser import PmidParser
from liyi_cute.shared.imports.schemas.schemaItem import TraningNerExampleSchema

obj = PmidParser(task_name="ner", error="ignore")
examples = obj.parse(r"../data/pmid")
#[{

# }]
t = TraningNerExampleSchema(many=True)
a = cut_sent(examples, max_length=256)
s = t.dump(a)
ex = t.load(s)
print(ex)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 22:10
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : test1.py

from collections import Counter

class Metrics(object):
    """用于评价模型，计算每个标签的精确率，召回率，F1分数"""

    def __init__(self, golden_tags, predict_tags, remove_O=False):

        # [[t1, t2], [t3, t4]...] --> [t1, t2, t3, t4...]
        self.golden_tags = flatten_lists(golden_tags)     #将已知标签和预测的标签拼成列表
        self.predict_tags = flatten_lists(predict_tags)

        if remove_O:  # 将O标记移除，只关心实体标记
            self._remove_Otags()

        # 辅助计算的变量
        self.tagset = set(self.golden_tags)    #标签集合（不重复）
        self.correct_tags_number = self.count_correct_tags()    #某个tag预测正确的次数
        self.predict_tags_counter = Counter(self.predict_tags)  #预测标签各个标签的个数
        self.golden_tags_counter = Counter(self.golden_tags)    #源文件标签各个标签的个数

        # 计算精确率
        self.precision_scores = self.cal_precision()

        # 计算召回率
        self.recall_scores = self.cal_recall()

        # 计算F1分数
        self.f1_scores = self.cal_f1()

    def cal_precision(self):
    #计算每个tag的准确率吧
        precision_scores = {}
        for tag in self.tagset:
            #correct_tags_number为tag计算正确的次数     predict_tags_counter所有predict_tag的个数
            precision_scores[tag] = self.correct_tags_number.get(tag, 0) / \
                                    max(1e-10, self.predict_tags_counter[tag])

        return precision_scores

    def cal_recall(self):
    #计算每个tag的召回率
        recall_scores = {}
        for tag in self.tagset:
            #correct_tags_number为tag计算正确的次数     golden_tags_counter为tag出现的总次数
            recall_scores[tag] = self.correct_tags_number.get(tag, 0) / \
                                 max(1e-10, self.golden_tags_counter[tag])
        return recall_scores

    def cal_f1(self):
        f1_scores = {}
        for tag in self.tagset:
            p, r = self.precision_scores[tag], self.recall_scores[tag]
            f1_scores[tag] = 2 * p * r / (p + r + 1e-10)  # 加上一个特别小的数，防止分母为0
        return f1_scores

    def report_scores(self):
        """将结果用表格的形式打印出来，像这个样子：

                      precision    recall  f1-score   support
              B-LOC      0.775     0.757     0.766      1084
              I-LOC      0.601     0.631     0.616       325
             B-MISC      0.698     0.499     0.582       339
             I-MISC      0.644     0.567     0.603       557
              B-ORG      0.795     0.801     0.798      1400
              I-ORG      0.831     0.773     0.801      1104
              B-PER      0.812     0.876     0.843       735
              I-PER      0.873     0.931     0.901       634

          avg/total      0.779     0.764     0.770      6178
        """
        # 打印表头
        header_format = '{:>9s}  {:>9} {:>9} {:>9} {:>9}'
        header = ['precision', 'recall', 'f1-score', 'support']
        print(header_format.format('', *header))

        row_format = '{:>9s}  {:>9.4f} {:>9.4f} {:>9.4f} {:>9}'
        # 打印每个标签的 精确率、召回率、f1分数
        for tag in self.tagset:
            print(row_format.format(
                tag,
                self.precision_scores[tag],
                self.recall_scores[tag],
                self.f1_scores[tag],
                self.golden_tags_counter[tag]
            ))

        # 计算并打印平均值
        avg_metrics = self._cal_weighted_average()
        print(row_format.format(
            'avg/total',
            avg_metrics['precision'],
            avg_metrics['recall'],
            avg_metrics['f1_score'],
            len(self.golden_tags)
        ))

    def count_correct_tags(self):
        """计算每种标签预测正确的个数(对应精确率、召回率计算公式上的tp)，用于后面精确率以及召回率的计算"""
        correct_dict = {}
        for gold_tag, predict_tag in zip(self.golden_tags, self.predict_tags):  #zip()用于将数据打包成元组
            if gold_tag == predict_tag:    #每当匹配的情况就将correct_dict[gold_tag]加一
                if gold_tag not in correct_dict:   #若该标签不在已知标签字典中，将其加入
                    correct_dict[gold_tag] = 1
                else:
                    correct_dict[gold_tag] += 1

        return correct_dict

    def _cal_weighted_average(self):

        weighted_average = {}
        total = len(self.golden_tags)   #标准标签的总数

        # 计算weighted precisions:
        weighted_average['precision'] = 0.
        weighted_average['recall'] = 0.
        weighted_average['f1_score'] = 0.
        for tag in self.tagset:
            size = self.golden_tags_counter[tag]  #标准文件各个标签的个数
            weighted_average['precision'] += self.precision_scores[tag] * size
            weighted_average['recall'] += self.recall_scores[tag] * size
            weighted_average['f1_score'] += self.f1_scores[tag] * size

        for metric in weighted_average.keys():
            weighted_average[metric] /= total

        return weighted_average

    def _remove_Otags(self):

        length = len(self.golden_tags)
        O_tag_indices = [i for i in range(length)
                         if self.golden_tags[i] == 'O']

        self.golden_tags = [tag for i, tag in enumerate(self.golden_tags)
                            if i not in O_tag_indices]

        self.predict_tags = [tag for i, tag in enumerate(self.predict_tags)
                             if i not in O_tag_indices]
        print("原总标记数为{}，移除了{}个O标记，占比{:.2f}%".format(
            length,
            len(O_tag_indices),
            len(O_tag_indices) / length * 100
        ))

    def report_confusion_matrix(self):
        """计算混淆矩阵"""

        print("\nConfusion Matrix:")
        tag_list = list(self.tagset)
        # 初始化混淆矩阵 matrix[i][j]表示第i个tag被模型预测成第j个tag的次数
        tags_size = len(tag_list)
        matrix = []
        for i in range(tags_size):
            matrix.append([0] * tags_size)

        # 遍历tags列表
        for golden_tag, predict_tag in zip(self.golden_tags, self.predict_tags):
            try:
                row = tag_list.index(golden_tag)
                col = tag_list.index(predict_tag)
                matrix[row][col] += 1
            except ValueError:  # 有极少数标记没有出现在golden_tags，但出现在predict_tags，跳过这些标记
                continue

        # 输出矩阵
        row_format_ = '{:>7} ' * (tags_size + 1)
        print(row_format_.format("", *tag_list))
        for i, row in enumerate(matrix):
            print(row_format_.format(tag_list[i], *row))


def flatten_lists(lists):
    """
    将列表的列表拼成一个列表
    :param lists:
    :return:
    """
    flatten_list = []
    for l in lists:
        if type(l) == list:
            flatten_list += l
        else:
            flatten_list.append(l)
    return flatten_list
print(sorted(["B", "I", "O", "E", "S"]))


# -*- coding:UTF-8 -*-

# author:user
# contact: test@test.com
# datetime:2022/3/28 13:56
# software: PyCharm

"""
文件说明：
    序列化与反序列化
"""
from liyi_cute.shared.imports.schemas.schemaItem import ExampleSchema

# 对象之间的序列化与反序列话
from liyi_cute.shared.imports.bart_parser import BratParser
brat = BratParser(task_name="rel", error="ignore")
examples = brat.parse("../data/bio/")
example_schema = ExampleSchema() # 也可以加入过滤参数 only=("mention", "start")

## 序列化
exs = example_schema.dump(examples, many=True)

## 反序列化
sez = example_schema.load(exs, many=True)


## 加载json 序列化
import json
js = json.load(open("../data/output/test.json", 'r', encoding="utf-8"))
se1 = example_schema.load(js, many=True)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 13:08
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : base_parser.py
from __future__ import annotations

import pickle
from typing import Optional, Text, Union, Dict, List
from collections import Iterable
import re, os, logging
from copy import deepcopy

from liyi_cute.shared.exceptions import NotImplementedException, NotExistException, FileNotFoundException, \
    NotSupportedException
from liyi_cute.shared.imports.schemas.schema import Example


class BasePaeser(object):
    exts = {".ann", ".txt", ".json"}
    support_task_name = {"ner", "rel" "env", "attr"}
    types = {"T", "R", "*", "E", "N", "AM"}
    supported_language_list = ["en", "zh"]
    support_save = {".pkl", ".json", "txt", "yml", "txt"}
    errors = {"raise", "ignore"}
    def __init__(self, task_name, ignore_types: Optional[Iterable[str]] = None, error: str = "raise", lang="en"):
        self.examples:List = []
        ##任务类型
        if task_name not in self.support_task_name:
            raise NotImplementedException("The task interface is not implemented")
        self.task_name = task_name

        ## 判断忽略的类型
        self.re_ignore_types: Optional[re.Pattern] = None
        if ignore_types:
            unknown_types = set(ignore_types) - self.types
            if unknown_types:
                raise NotExistException(f"Unknown types: {unknown_types!r}")
            ## 忽略类型正则
            self.re_ignore_types = re.compile(r"|".join(re.escape(x) for x in ignore_types))

        ##
        if error not in self.errors:
            raise NotExistException(f"`error` should be in {self.errors!r}")
        self.error = error

        ## 支持语言
        if lang not in self.supported_language_list:
            raise NotExistException(f"{lang} is not support")
        self.lang = lang

    def _should_ignore_line(self, line:Text):
        if self.re_ignore_types:
            return re.match(self.re_ignore_types, line)

        return False

    def _raise(self, error)->None:
        if self.error == "raise":
            raise error

    def _raise_invalid_line_error(self, line:Text)->None:
        self._raise(NotExistException(f"Invalid line: {line}"))


    def save_example(self, examples: List[Example], output: Text) -> None:
        file_ext = os.path.splitext(output)[1]
        if not os.path.exists(os.path.dirname(output)):
            raise FileNotFoundException(f"{os.path.dirname(output)}:path not exits")

        if file_ext not in self.support_save:
            raise NotSupportedException

        with open(output, "wb") as f:
            pickle.dump(examples, f)

    def load_example(self, input: Text) -> List[Example]:
        with open(input, "rb") as f:
            examples = pickle.load(f)

        return examples

    def parse(self,  dirname: Union[str, bytes, os.PathLike], encoding: str = "utf-8"):
        raise NotImplementedError

    @staticmethod
    def filter_sample(examples, keep_content:List, filter_type="ner"):
        if not isinstance(keep_content, List):
            raise ValueError

        if filter_type=="ner":
            new_examples = deepcopy(examples)
            for example in new_examples:
                entities = []
                for ent in example.entities:
                    if ent.type in keep_content:
                        entities.append(ent)
                example.entities = entities
            return new_examples

    def entities(self):
        entities_set = self.entity_type()
        entities = {key:set() for key in entities_set}

        for example in self.examples:
            for ent in example.entities:
                entities[ent.type].add(ent.mention)
        return entities

    def _tag_type(self, all_tag:List[List]):
        if not (isinstance(all_tag, list) and isinstance(all_tag[0], list)):
            raise ValueError
        tag_type = "".join(sorted(list(set([tag.split("-")[0] for tags in all_tag for tag in tags]))))
        if tag_type == "BIO":
            return "BIO"
        elif tag_type == "BEIOS":
            return "BEIOS"
        else:
            raise NotImplementedError

    def entity_type(self):
        entities_set = set()
        for example in self.examples:
            for ent in example.entities:
                entities_set.add(ent.type)
        return entities_set

    def relations(self):
        raise NotImplementedError

    def relation_type(self):
        raise NotImplementedError

    @classmethod
    def create(cls, **kwargs):
        task_name = kwargs.pop("task_name", None)
        ignore_types = kwargs.pop("ignore_types", None)
        error = kwargs.pop("error", "raise")

        return cls(task_name=task_name,
                   ignore_types=ignore_types,
                   error=error)









# -*- coding:UTF-8 -*-

# author:user
# contact: test@test.com
# datetime:2022/4/1 17:03
# software: PyCharm

"""
文件说明：
    
"""
from __future__ import annotations

from typing import Tuple, List, Dict, Text
import logging

from tqdm import tqdm

from liyi_cute.shared.imports.schemas.schema import Example, Entity, TraningNerExample, TraningExample

logger = logging.getLogger(__file__)

def cut_sentences_v1(sent_tupe:Tuple,  punctuation_special_char={".", ";"})->List[Tuple]:
    """
    [(0, "1111"), (65, "222")], tupe第一位是原文的开始位置，第二位是原文截断后的位置
    """
    init_start = sent_tupe[0]
    sent = sent_tupe[1]
    sents = []
    string = ""
    start = 0
    for index, char in enumerate(sent):
        if char==" " and index-1>0 and (sent[index-1] in punctuation_special_char):
            string +=sent[index]
            sents.append((init_start+start, string))
            string = ""
            start = index+1
        else:
            string += sent[index]
    if string:
        sents.append((init_start+start, string))
    return sents

def cut_sentences_v2(sent_tupes:List[Tuple], entities:Entity)->List[Tuple]:
    new_sent_tupes = []
    for sent_tupe in sent_tupes:
        init_start = sent_tupe[0]
        c_text = sent_tupe[1]
        mid = len(c_text)//2
        entities = sorted(entities, key=lambda x: x.start)
        while True:
            flag = False
            for ent in entities:
                if ent.start<=mid <ent.end:
                    flag = True

            if not flag or mid>=len(c_text):
                break
            mid +=1
        new_sent_tupes+=[(init_start, c_text[:mid]),(init_start+mid, c_text[mid:])]
    return new_sent_tupes

def correct_tag_position(all_cut_sents:List[Tuple], annotaion:List[Entity])->List[Dict]:
    """
    [Entity(start=0, end=4, mention="KRAS", type="GENE", id="T1"),...]
    return [{'id': 0, 'text': 'KRAS G12V mutation', 'entities': [[0, 4, 'KRAS', 'GENE'], [5, 9, 'G12V', 'DESEASE']]}]
    """
    all_data = []
    sent_ids = 0
    for c_s in all_cut_sents:
        cut_text = c_s[1]
        cut_start = c_s[0]
        cut_end = cut_start + len(cut_text)
        entities = []
        for an in annotaion:
            e_start = an.start
            e_end = an.end
            if cut_start <=e_start< cut_end and cut_start <=e_end< cut_end:
                entities.append([e_start-cut_start, e_end-cut_start, an.mention, an.type])
        all_data.append({"id":sent_ids, "text":cut_text, "entities":entities})
        sent_ids += 1
    return all_data

def cut_sent(examples:List[Example], max_length:int=512)->List[TraningExample]:
    # 将句子分句，细粒度分句后再重新合并
    """

    :param examples:
    :param max_length:
    :return: [{"text":"", "entities":[(0,4,"KRAS", "GENE"), (5,9,"G12V", "DESEASE")], "cut_text":[{'id': 0, 'text': 'KRAS G12V mutation', 'entities': [[0, 4, 'KRAS', 'GENE'], [5, 9, 'G12V', 'DESEASE']]}]}]
    """
    sentences = []

    ## 以句号为分割符号
    tq_bar = tqdm(examples, desc="cut example sentence")
    train_samples = []
    for index, example in enumerate(tq_bar):
        tq_bar.set_description("example number: %s" %index)
        if example.task_name=="ner":
            text = example.text
            entities = example.entities # [Entity()]
            cut_sents_list = [(0, text)]
            ## 一阶截断
            if is_ge_max_len(cut_sents_list, max_length):
                cut_sents_list = cut_sentences_v1((0, example.text), punctuation_special_char={"."}) #[(0, "1111"), (65, "222")]
            ## 二阶截断
            if is_ge_max_len(cut_sents_list, max_length):
                two_cut_sents_list = cut_sents_list
                cut_sents_list = []
                for cut_sent in two_cut_sents_list:
                    if len(cut_sent[1])>max_length:
                        cut_sents_list+=cut_sentences_v2([cut_sent], entities)
                    else:
                        cut_sents_list+=[cut_sent]

            ## 截断后纠正标签的起始和结束的位置
            correct_cut_sents_list = correct_tag_position(cut_sents_list, entities) # [{'id': 0, 'text': 'KRAS G12V mutation', 'entities': [[0, 4, 'KRAS', 'GENE'], [5, 9, 'G12V', 'DESEASE']]}]

            ## 校验标签是否对齐
            check_ner_postion(correct_cut_sents_list)
            for idx, ccsl in enumerate(correct_cut_sents_list):
                train_samples.append(TraningNerExample(id=example.id,
                                                       task_name=example.task_name,
                                                       text=text,
                                                       entities= example.entities,
                                                       sub_id=idx,
                                                       cut_text=ccsl['text'],
                                                       cut_entities=[Entity(id=str(index), start=cu_ent[0],
                                                                            end=cu_ent[1], mention=cu_ent[2],
                                                                            type=cu_ent[3])
                                                                     for index, cu_ent in enumerate(ccsl['entities'])],
                                                       cut_number=len(correct_cut_sents_list),
                                                       cut_len=len(ccsl['text']))
                                     )
        else:
            raise NotImplementedError

    return train_samples

def is_ge_max_len(sents_list:List[Tuple], max_len:int)->bool:
    for cut_sent in sents_list:
        if len(cut_sent[1])>max_len:
            return True
    return False


def check_ner_postion(correct_cut_sent_list:List[Dict])->None:
    """
    [{'id': 0, 'text': 'KRAS G12V mutation', 'entities': [[0, 4, 'KRAS', 'GENE'], [5, 9, 'G12V', 'DESEASE']]}]
    :param correct_cut_sent_list:
    :return:
    """
    for one_cut_sent in correct_cut_sent_list:
        cut_text = one_cut_sent["text"]
        for ent in one_cut_sent['entities']:
            mention = ent[2]
            cut_start = ent[0]
            cut_end =  ent[1]
            if cut_text[cut_start:cut_end]!=mention:
                logger.warning("entity is not aligned")
                logger.warning("cut_text:%s" % cut_text)
                logger.warning("cut_start:%s" %cut_start)
                logger.warning("cut_end:%s" % cut_end)
                logger.warning("mention:%s" % mention)
                logger.warning("cut tag:%s" % cut_text[cut_start:cut_end])
                raise ValueError




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 14:56
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : common.py

from __future__ import annotations

import copy
import glob
import itertools
import os
import logging

from collections.abc import Iterable
from typing import Union, Optional, Dict, Text, Any, List


def iter_file_groups(
    dirname: Union[str, bytes, os.PathLike],
    exts: Union[str, Iterable[str]],
    missing: str = "error",
) -> Iterable[tuple[str, list[str]]]:
    def _format_ext(ext):
        return f'.{ext.lstrip(".")}'

    def _iter_files(dirname):
        for dirpath, _, filenames in os.walk(dirname):
            for filename in filenames:
                if os.path.splitext(filename)[1] in exts:
                    yield os.path.join(dirpath, filename)

    if isinstance(dirname, bytes):
        dirname = dirname.decode()  # type: ignore

    missings = {"error", "ignore"}
    if missing not in missings:
        raise ValueError(f"Param `missing` should be in {missings}")

    if isinstance(exts, str):
        return glob.iglob(
            os.path.join(dirname, f"**/*{_format_ext(exts)}"), recursive=True
        )

    exts = {*map(_format_ext, exts)}
    num_exts = len(exts)
    files = _iter_files(dirname)
    for key, group in itertools.groupby(
        sorted(files), key=lambda x: os.path.splitext(os.path.relpath(x, dirname))[0]
    ):
        sorted_group = sorted(group, key=lambda x: os.path.splitext(x)[1])
        if len(sorted_group) != num_exts and missing == "error":
            raise RuntimeError(f"Missing files: {key!s}.{exts}")

        yield key, sorted_group


def override_defaults(
    defaults: Optional[Dict[Text, Any]], custom: Optional[Dict[Text, Any]]
) -> Dict[Text, Any]:
    """Override default config with the given config.
    code  from rasa 2.4.1
    We cannot use `dict.update` method because configs contain nested dicts.

    Args:
        defaults: default config
        custom: user config containing new parameters

    Returns:
        updated config
    """
    if defaults:
        config = copy.deepcopy(defaults)
    else:
        config = {}

    if custom:
        for key in custom.keys():
            if isinstance(config.get(key), dict):
                config[key].update(custom[key])
            else:
                config[key] = custom[key]

    return config


def init_logger(log_file=None, log_file_level=logging.NOTSET):
    """
    :param log_file: str, 文件目录
    :param log_file_level: str, 日志等级
    """
    log_format = logging.Formatter(fmt='%(asctime)s - %(levelname)s - %(name)s -   %(message)s',
                                   datefmt='%m/%d/%Y %H:%M:%S')
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(log_format)
    logger.handlers = [console_handler]
    if log_file and log_file != '':
        file_handler = logging.FileHandler(log_file,encoding="utf-8")
        file_handler.setLevel(log_file_level)
        logger.addHandler(file_handler)
    return logger

def check_bio(tags:List)->bool:
    """
    检测输入的tags是否是bio编码
    如果不是bio编码
    那么错误的类型
    (1)编码不在BIO中
    (2)第一个编码是I
    (3)当前编码不是B,前一个编码不是O
    :param tags:
    :return:
    """
    for i, tag in enumerate(tags):
        if tag == 'O':
            continue
        tag_list = tag.split("-")
        if len(tag_list) != 2 or tag_list[0] not in set(['B','I']):
            #非法编码
            return False
        if tag_list[0] == 'B':
            continue
        elif i == 0 or tags[i-1] == 'O':
            #如果第一个位置不是B或者当前编码不是B并且前一个编码0，则全部转换成B
            tags[i] = 'B' + tag[1:]
        elif tags[i-1][1:] == tag[1:]:
            # 如果当前编码的后面类型编码与tags中的前一个编码中后面类型编码相同则跳过
            continue
        else:
            # 如果编码类型不一致，则重新从B开始编码
            tags[i] = 'B' + tag[1:]
    return True


def bio_to_bioes(tags:List)->List:
    """
    把bio编码转换成bioes编码
    返回新的tags
    :param tags:
    :return:
    """
    new_tags = []
    for i, tag in enumerate(tags):
        if tag == 'O':
            # 直接保留，不变化
            new_tags.append(tag)
        elif tag.split('-')[0] == 'B':
            # 如果tag是以B开头，那么我们就要做下面的判断
            # 首先，如果当前tag不是最后一个，并且紧跟着的后一个是I
            if (i+1) < len(tags) and tags[i+1].split('-')[0] == 'I':
                # 直接保留
                new_tags.append(tag)
            else:
                # 如果是最后一个或者紧跟着的后一个不是I，那么表示单子，需要把B换成S表示单字
                new_tags.append(tag.replace('B-','S-'))
        elif tag.split('-')[0] == 'I':
            # 如果tag是以I开头，那么我们需要进行下面的判断
            # 首先，如果当前tag不是最后一个，并且紧跟着的一个是I
            if (i+1) < len(tags) and tags[i+1].split('-')[0] == 'I':
                # 直接保留
                new_tags.append(tag)
            else:
                # 如果是最后一个，或者后一个不是I开头的，那么就表示一个词的结尾，就把I换成E表示一个词结尾
                new_tags.append(tag.replace('I-', 'E-'))

        else:
            raise Exception('非法编码')
    return new_tags

def bioes_to_bio(tags:List)->List:
    """
    BIOES->BIO
    :param tags:
    :return:
    """
    new_tags = []
    for i, tag in enumerate(tags):
        if tag.split('-')[0] == "B":
            new_tags.append(tag)
        elif tag.split('-')[0] == "I":
            new_tags.append(tag)
        elif tag.split('-')[0] == "S":
            new_tags.append(tag.replace('S-','B-'))
        elif tag.split('-')[0] == "E":
            new_tags.append(tag.replace('E-','I-'))
        elif tag.split('-')[0] == "O":
            new_tags.append(tag)
        else:
            raise Exception('非法编码格式')
    return new_tags






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 12:20
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : lm_tokenizer.py
from __future__ import annotations

from typing import Optional, Dict, Text, Any, List

from transformers import AutoTokenizer

from liyi_cute.processor.tokenizers.tokenizer import Tokenizer
from liyi_cute.shared.exceptions import ParamNotExistException
from liyi_cute.shared.imports.schemas.schema import TraningExample


class LanguageModelTokenizer(Tokenizer):
    defaults = {
        "pretrained_model_name_or_path": "bert-base-uncased",
        "add_special_tokens": True,
        "max_length": 512,
        "padding":"max_length",
        "return_offsets_mapping":True
    }
    keep_keys = ["input_ids", "token_type_ids", "attention_mask", "offset_mapping","sequence_ids"]

    def __init__(self, component_config: Optional[Dict[Text, Any]]):
        super().__init__(component_config)
        self.task_name = self.component_config.get("task_name", None)
        if self.task_name is None:
            raise ParamNotExistException("miss parameter task_name")

        self.tokenizer = AutoTokenizer.from_pretrained(self.component_config.get("pretrained_model_name_or_path"))

    def tokenize(self, train_example: Optional[TraningExample]):
        encoded = self.process(text=train_example.cut_text)
        alignment_tags = self.label_alignment(encoded, train_example.cut_entities)
        train_example.true_ner_tag = alignment_tags

        for key in self.keep_keys:
            setattr(train_example, key, encoded[key])

        return train_example

    def process(self, text:Text)->Dict:
        ## key ['input_ids', 'token_type_ids', 'attention_mask', 'offset_mapping']
        encoded = self.tokenizer(text,
                                add_special_tokens=self.component_config.get("add_special_tokens"),
                                max_length=self.component_config.get("max_length"),
                                padding=self.component_config.get("padding"),
                                return_offsets_mapping=self.component_config.get("return_offsets_mapping"))

        keep_encoded = {key: encoded[key] for key in encoded}
        if "sequence_ids" in self.keep_keys:
            keep_encoded["sequence_ids"] = encoded.sequence_ids()
        return keep_encoded



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 10:05
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : tokenizer.py
from __future__ import annotations

from typing import Optional, Dict, Any, Text, List

from liyi_cute.processor.component import Component
from liyi_cute.shared.imports.schemas.schema import Entity


class Tokenizer(Component):
    # {"input_ids":[], "token_type_ids":[], "attention_mask":[], "offset_mapping":[]}
    keep_keys = ["input_ids", "token_type_ids", "attention_mask", "offset_mapping"]
    def __init__(self,component_config: Optional[Dict[Text, Any]] = None):
        super().__init__(component_config)
        self.component_config = component_config

    def tokenize(self, text):

        raise NotImplementedError

    def label_alignment(self, encoded:Dict, annotaion:List[Entity], start_position_id=0, end_postion_id=0) -> List[Text]:
        offset_mapping = encoded['offset_mapping']
        squence_ids = encoded['sequence_ids']
        label = ["O"] * len(offset_mapping)
        for label_ids, loc in enumerate(annotaion):
            start_char = int(loc.start)
            end_char = int(loc.end)
            label_text = loc.mention
            tag_text = loc.type

            # token start index
            token_start_index = 0
            while squence_ids[token_start_index] != start_position_id:
                token_start_index += 1
            # token end index
            token_end_index = len(offset_mapping) - 1
            while squence_ids[token_end_index] != end_postion_id:
                token_end_index -= 1

            while token_start_index < len(offset_mapping) and offset_mapping[token_start_index][0] <= start_char:
                token_start_index += 1
            token_start_index -= 1

            while offset_mapping[token_end_index][1] >= end_char:
                token_end_index -= 1
            token_end_index += 1
            label[token_start_index] = "B-" + tag_text
            for idt in range(token_start_index + 1, token_end_index + 1):
                label[idt] = "I-" + tag_text
        return label





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 13:23
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : component.py
from typing import Optional, Dict, Text, Any

from liyi_cute.utils.common import override_defaults


class Component(object):
    defaults = {}
    def __init__(self, component_config: Optional[Dict[Text, Any]] = None):

        if not component_config:
            component_config = {}

        self.component_config = override_defaults(
            self.defaults, component_config
        )


    @classmethod
    def load(cls):
        raise NotImplementedError

    @classmethod
    def create(cls, component_config: Dict[Text, Any],):
        return cls(component_config)

    def persist(self, file_name: Text, model_dir: Text) -> Optional[Dict[Text, Any]]:
        """Persists this component to disk for future loading.

        Args:
            file_name: The file name of the model.
            model_dir: The directory to store the model to.

        Returns:
            An optional dictionary with any information about the stored model.
        """
        pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 12:23
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : liyi_tokenizer.py
from __future__ import annotations

from liyi_cute.processor.tokenizers.tokenizer import Tokenizer


class LiyiTokenizer(Tokenizer):
    def __init__(self):
        pass


# -*- coding:UTF-8 -*-

# author:user
# contact: test@test.com
# datetime:2022/3/29 16:40
# software: PyCharm

"""
文件说明：
    
"""
from __future__ import annotations
from typing import Dict

from spacy import displacy
import spacy


from liyi_cute.shared.exceptions import NotExistException
from liyi_cute.utils.constants import GRADIENTS

def spacy_visualize(doc:Dict, options={}, style="dep",
                    manual=False, jupyter=False, mode="html",
                    lang="en_core_web_sm"):
    """
    :param doc:
    ent example: {"text": "But Google is starting from behind.", "ents": [{"start": 4, "end": 10, "label": "ORG"],"title": None}
    dep example: dep = {"words": [{"text": "This", "tag": "DT"}, {"text": "is", "tag": "VBZ"}], "arcs": [{"start": 0, "end": 1, "label": "nsubj", "dir": "left"}]}
    :param options:
    :param style:
    :param manual: Don't parse `Doc` and instead expect a dict/list of dicts.
    :param jupyter:
    :param mode: html, serve
    :param mode: lang
    :return:
    """
    spacy.load(lang)

    if not options:
        options = spacy_options(doc)

    if mode=="html":
        return displacy.render(doc, style=style, options=options,
                               manual=manual, jupyter=jupyter)

    displacy.serve(doc, style=style, manual=manual, options=options)

def spacy_options(doc:Dict)->Dict:
    """
    :param doc:
    :return:
    """
    labels = set()
    ents = doc.get("ents", [])
    for en in ents:
        if "label" not in en:
            raise NotExistException(f"key error: label")
        labels.add(en.get("label"))
    op_labels = []
    op_colors = {}

    for index, label in enumerate(labels):
        idx = index%len(labels)
        op_labels.append(label.upper())
        op_colors.update({label.upper():GRADIENTS[idx]})

    return {"ents": op_labels, "colors": op_colors}

def check_doc(doc, style="dep"):
    pass



# -*- coding:UTF-8 -*-

# author:user
# contact: test@test.com
# datetime:2022/3/29 12:31
# software: PyCharm

"""
文件说明：
    
"""
from __future__ import annotations

from typing import Optional, Union, Text, List, Dict, Tuple
from collections.abc import Iterable
import dataclasses

"""
#################################### 加载文件中间数据 ##################################
"""
@dataclasses.dataclass
class Entity(object):
    mention: str
    type: str
    start: int
    end: int
    id: Optional[str] = None

@dataclasses.dataclass
class Relation(object):
    type: str
    arg1: Entity
    arg2: Entity
    id: Optional[str] = None

@dataclasses.dataclass
class Event(object):
    id: Optional[str] = None

@dataclasses.dataclass
class Example(object):
    text: Union[str, Iterable[str]]
    entities: list[Entity] = dataclasses.field(default_factory=list)
    relations: list[Relation] = dataclasses.field(default_factory=list)
    events: list[Event] = dataclasses.field(default_factory=list)
    id: Optional[str] = None
    task_name:Optional[str] = None

"""
############################### 预处理中间数据 ################################
"""
# {'sub_id': 0, 'cut_text': 'KRAS G12V mutation',
# 'cut_entities': [[0, 4, 'KRAS', 'GENE']]}]}

@dataclasses.dataclass
class TraningExample(object):
    text:Text
    cut_text: Text
    sub_id: int
    cut_number: int = dataclasses.field(default_factory=0)
    entities:List[Entity] = dataclasses.field(default_factory=list)
    cut_entities:List[Entity] = dataclasses.field(default_factory=list)
    cut_len: int = dataclasses.field(default_factory=0)
    id: Optional[str] = None
    task_name:Optional[str] = None
    input_ids: List[int] = dataclasses.field(default_factory=list)
    attention_mask: List[int] = dataclasses.field(default_factory=list)
    token_type_ids: List[int] = dataclasses.field(default_factory=list)

@dataclasses.dataclass
class TraningNerExample(TraningExample):
    true_ner_tag: List[str] = dataclasses.field(default_factory=list)
    pre_ner_tag: List[str] = dataclasses.field(default_factory=list)
    offset_mapping:List[Tuple] = dataclasses.field(default_factory=list)

"""
###################################### 可视化 #################################
"""
@dataclasses.dataclass
class VisEnt(object):
    start:int
    end:int
    label:Text

@dataclasses.dataclass
class VisEntExample(object):
    text: Text
    ents: List[VisEnt] = dataclasses.field(default_factory=list)
    title:str = None

@dataclasses.dataclass
class VisDepWord(object):
    text:str
    tag:str

@dataclasses.dataclass
class VisDepArc(object):
    start:int
    end:int
    label:str
    dir:str

@dataclasses.dataclass
class VisDepExample(object):
    words:List[VisDepWord] = dataclasses.field(default_factory=list)
    arcs:List[VisDepArc] = dataclasses.field(default_factory=list)

@dataclasses.dataclass
class VisExample(object):
    visent:VisEntExample
    visdep:VisDepExample
    id: str = None







#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/29 20:33
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : example_visualize.py
from liyi_cute.vis.visualization import spacy_visualize
import spacy
spacy.load("en_core_web_sm")

if __name__ == '__main__':

    # doc = {'text': 'ALK-rearranged squamous cell lung carcinoma responding to crizotinib: A missing link in the field of non-small cell lung cancer? ALK-rearrangements are mainly encountered in lung adenocarcinomas and allow treating patients with anti-ALK targeted therapy. ALK-rearranged squamous cell lung carcinomas are rare tumors that can also respond to anti-ALK-targeted therapy. Nevertheless, ALK screening is not always performed in patients with squamous cell lung carcinomas making the identification and treatment of this molecular tumor subtype challenging. We intend to report a rare case of ALK-rearranged lung squamous cell carcinoma with response to crizotinib therapy. We report clinical, pathological, immunohistochemical and fluorescent in situ hybridization data concerning a patient having an ALK-rearranged squamous cell lung cancer diagnosed in our institution. The patient was a 58-year old woman with a metastatic-stage lung cancer. Histopathological and immunohistochemical analyses were performed on a bronchial biopsy sample and concluded in a non-keratinizing squamous cell lung carcinoma expressing strongly cytokeratin 5/6, p63 and p40, which are classic hallmarks of lung squamous cell carcinomas, but also cytokeratin 7 which is more commonly expressed in lung adenocarcinomas. The tumor did not express thyroid transcription factor-1. ALK rearrangement was searched because of the never-smoker status of the patient and resulted in strong positive fluorescent in situ hybridization test and ALK/p80 immunohistochemistry. The patient responded to crizotinib therapy during 213 days. Our observation points out the interest of considering ALK screening in patients with metastatic lung squamous cell carcinomas, especially in patients lacking a usual heavy-smoker clinical history. The histopathological and immunohistochemical features of this particular tumor highlighting the overlapping criteria between lung adenocarcinomas and rare ALK-rearranged squamous cell lung carcinomas could also be relevant to extend ALK screening to tumors with intermediate phenotypes between squamous cell carcinomas and adenocarcinomas and/or arising in non-smokers.', 'ents': [{'start': 0, 'end': 3, 'label': 'Gene'}]}
    #
    # html = spacy_visualize(doc, style="ent", manual=True, mode="img", output_path="001.svg")
    # print(1)
    import spacy
    from spacy import displacy
    from pathlib import Path

    nlp = spacy.load("en_core_web_sm")
    sentences = ["When Sebastian Thrun started working on self-driving cars at Google in 2007, few people outside of the company took him seriously."]
    for sent in sentences:
        doc = nlp(sent)
        svg = displacy.render(doc, style="ent", jupyter=False)
        file_name = "111.svg"
        output_path = Path(file_name)
        output_path.open("w", encoding="utf-8").write(svg)



# -*- encoding: utf-8 -*-
'''
Filename         :main.py
Description      :
Time             :2022/03/26 00:32:41
Author           :daiyizheng
Email            :387942239@qq.com
Version          :1.0
'''

# 关系标注
# ann 转 json
## ann 中的数据
import json
import os
import random
import logging

from tqdm import tqdm

logger = logging.getLogger(__file__)


class PubTatorParseData(object):
    def __init__(self) -> None:
        self.alphabet = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
                         't', 'u', 'v', 'w', 'x', 'y', 'z',
                         'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
                         'T', 'U', 'V', 'W', 'X', 'Y', 'Z']
        self.digits = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']
        self.special_symbols = ["(", ")", ",", ".", "/", ">", "<", ";", "-"]

    def check_annotations(self, train_texts, infomations) -> bool:
        for an in infomations.values():
            start_id = an[0]
            end_ids = start_id + an[1]
            if train_texts[start_id:end_ids] != an[2]:
                logger.warning("start_ids: " + str(start_id))
                logger.warning("end_ids: " + str(end_ids))
                logger.warning("text content: " + train_texts[start_id:end_ids])
                logger.warning("annotations content: " + an[2])
                return False
        return True

    def parse_biocjson(self, input_path, output_path):
        json_data_list = os.listdir(input_path)
        for index, file_name in tqdm(enumerate(json_data_list)):
            output_file_name = os.path.join(output_path, file_name.split(".")[0] + ".txt")
            text, infomations = self.load_json(os.path.join(input_path, file_name))
            is_aligned_tag = self.check_annotations(text, infomations)
            ## 文本和实体是否对齐
            if is_aligned_tag:
                all_tokens, all_tags = self.__json2txt(text, infomations)
                self.save_file(all_tokens, all_tags, output_file_name)
            else:
                logger.warning("文本与实体未对齐：" + file_name)

    def load_json(self, input_path):
        data = json.load(open(input_path, 'r', encoding="utf-8"))
        text = ""
        infomations = {}
        for item in data["passages"]:
            if item["text"]:
                text = text + item["text"] + " "
                for anns in item["annotations"]:
                    if anns["text"].strip() != "":
                        infomations.update({anns["locations"][0]["offset"]: [anns["locations"][0]["offset"],
                                                                             anns["locations"][0]["length"],
                                                                             anns["text"], anns["infons"]["type"]]})
        text = text.rstrip()
        return text, infomations

    def __json2txt(self, text, infomations):
        idx = 0
        all_tokens = []
        all_tags = []
        while (idx < len(text)):
            ## 标注信息
            if idx in infomations:
                start = infomations[idx][0]
                end = start + infomations[idx][1]
                type_name = infomations[idx][3]
                tokens_list = text[start:end].split(" ")
                tags_list = ["B-" + type_name]
                for _ in tokens_list[1:]:
                    tags_list.append("I-" + type_name)
                all_tokens += tokens_list
                all_tags += tags_list
                idx = end
            else:
                ## 空格直接换行
                if text[idx] == " ":
                    idx = idx + 1
                elif text[idx] in self.alphabet:
                    i = idx
                    idx = idx + 1
                    ## 纯字母xxx 或者 多字母拼接xx-xx 或者字母和数字拼接xx123
                    while (idx < len(text) - 1 and (
                            text[idx] in self.alphabet or text[idx] == "-" or text[idx] in self.digits)):
                        idx = idx + 1
                    all_tokens.append(text[i:idx])
                    all_tags.append("O")
                else:
                    # 如果是数字或者小数点数字
                    if text[idx].isdigit():
                        i = idx
                        idx = idx + 1
                        while (idx < len(text) - 1 and (text[idx].isdigit() or (
                                idx < len(text) - 2 and text[idx] == "." and text[idx + 1] != " "))):
                            idx = idx + 1
                        all_tokens.append(text[i:idx])
                        all_tags.append("O")
                    ## 句子结束. 后面多加一个加换行
                    if text[idx] == "." and idx < len(text) - 2 and text[idx + 1] == " ":
                        all_tokens.append(text[idx])
                        all_tags.append("O")
                        idx = idx + 1
                        ## 换行
                        all_tokens.append("\n")
                        all_tags.append("\n")
                    else:
                        all_tokens.append(text[idx])
                        all_tags.append("O")
                        idx = idx + 1
        assert len(all_tokens) == len(all_tags), "tokens length is not equal tags length"
        return all_tokens, all_tags

    def save_file(self, all_tokens, all_tags, output_path):
        with open(output_path, "w", encoding="utf-8") as fw:
            for tn, tg in zip(all_tokens, all_tags):
                if tn != "\n":
                    fw.write(tn + "\t" + tg)
                    fw.write("\n")
                else:
                    fw.write("\n")

    def split_data(self, input_path, shuffle=True, split_rate=0.95):
        all_tokens, all_tags = self.load_token_tag(input_path)
        all_content = list(zip(all_tokens, all_tags))
        n_total = len(all_content)
        offset = int(n_total * split_rate)
        if n_total == 0 or offset < 1:
            return all_content, []
        if shuffle:
            random.shuffle(all_content)
        return all_content[:offset], all_content[offset:]

    def load_token_tag(self, input_path, format_type="txt"):
        file_list = os.listdir(input_path)
        file_path_list = [os.path.join(input_path, file_name) for file_name in file_list if
                          file_name.endswith(format_type)]
        all_tokens = []
        all_tags = []
        for p in tqdm(file_path_list):
            tokens, tags = self.__read_txt(p)
            all_tokens.extend(tokens)
            all_tags.extend(tags)

        return all_tokens, all_tags

    def __read_txt(self, input_path):
        all_tokens = []
        all_tags = []
        per_file_tokens = []
        per_file_tags = []
        with open(input_path, 'r', encoding="utf-8") as fr:
            lines = fr.readlines()
            for line in lines:
                line = line.strip()
                if line:
                    line_list = line.split("\t")
                    if len(line_list) == 2:
                        per_file_tokens.append(line_list[0])
                        per_file_tags.append(line_list[1])
                else:
                    if per_file_tokens:
                        all_tokens.append(per_file_tokens)
                        all_tags.append(per_file_tags)
                    per_file_tokens = []
                    per_file_tags = []
        return all_tokens, all_tags

    def save_json(self, data):
        pass


if __name__ == '__main__':
    ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
    DATA_PATH = os.path.join(ROOT_PATH, "../../", "datasets", "pubtator_dataset", "output", "pmcid")
    ob = PubTatorParseData()
    ob.parse_biocjson(DATA_PATH, DATA_PATH)
    # train, valid = ob.split_data(DATA_PATH)
    print(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/4 15:18
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : standard_json_parser.py
from __future__ import annotations

import os
from typing import Optional, Iterable, Union

from liyi_cute.shared.imports.base_parser import BasePaeser


class StandardJsonPaeser(BasePaeser):
    def __init__(self,task_name, ignore_types: Optional[Iterable[str]] = None, error: str = "raise"):
        super().__init__(task_name,ignore_types, error)


    def parse(self, dirname: Union[str, bytes, os.PathLike], encoding: str = "utf-8", lang="en"):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 22:13
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : test3.py

class Metrics(object):
    """用于实体级别评价模型，计算每个标签的精确率，召回率，F1分数"""

    def __init__(self, std_tags, predict_tags):
        """
        初始化对照文件中的标签列表、预测文件的标签列表、以及
        :param std_tags:
        :param predict_tags:
        """
        #将按句的标签列表转化成按字的标签列表 如 [[t1, t2], [t3, t4]...] --> [t1, t2, t3, t4...]
        self.std_tags = flatten_lists(std_tags)     #将已知标签和预测的标签拼成列表
        self.predict_tags = flatten_lists(predict_tags)

        #计数器
        self.std_entity_counter = self.count_entity_dict(self.std_tags)  #标准结果的各实体个数
        self.predict_entity_counter = self.count_entity_dict(self.predict_tags)  #预测结果的各实体个数
        print("标准各实体个数", self.std_entity_counter)
        print("预测各实体个数", self.predict_entity_counter)

        self.std_entity_number = self.count_entity(self.std_tags)    #标准结果的实体总个数
        # self.predict_entity_number = self.count_entity(self.predict_tags)  #预测结果的实体总个数
        print("标准实体数", self.std_entity_number)
        # print("预测实体数", self.predict_entity_number)

        self.corrent_entity_number = self.count_correct_entity()
        print("正确的实体", self.corrent_entity_number)

        self.entity_set = set(self.std_entity_counter)
        print("实体集合", self.entity_set)

        # 计算精确率
        self.precision_scores = self.cal_precision()
        print("各个实体的准确率", self.precision_scores)

        # 计算召回率
        self.recall_scores = self.cal_recall()
        print("各个实体的召回率", self.recall_scores)

        # 计算F1分数
        self.f1_scores = self.cal_f1()
        print("各个实体的f1值", self.f1_scores)

        # 计算加权均值
        self.wighted_average = self._cal_wighted_average()
        print("各项指标的加权均值", self.wighted_average)

    def cal_precision(self):
    #计算每类实体的准确率
        precision_scores = {}
        #某实体正确的个数  /  预测中某实体所有的个数
        for entity in self.entity_set:                                                              #这里计算结果可能有问题
            precision_scores[entity] = self.corrent_entity_number.get(entity, 0) / max(1e-10, self.predict_entity_counter[entity])
        return precision_scores

    def cal_recall(self):
    #计算每类尸体的召回率
        recall_scores ={}
        for entity in self.entity_set:
            recall_scores[entity] = self.corrent_entity_number.get(entity, 0) / max(1e-10, self.std_entity_counter[entity])
        return recall_scores


    def cal_f1(self):
    #计算f1值
        f1_scores = {}
        for entity in self.entity_set:
            p, r = self.precision_scores[entity], self.recall_scores[entity]
            f1_scores[entity] = 2 * p * r / (p + r + 1e-10)
        return f1_scores

    def report_scores(self):
        """
        将结果用表格的形式打印出来
        :return:
        """
        # 打印表头
        header_format = '{:>9s}  {:>9} {:>9} {:>9} {:>9}'
        header = ['precision', 'recall', 'f1-score', 'support']
        print(header_format.format('', *header))
        row_format = '{:>9s}  {:>9.4f} {:>9.4f} {:>9.4f} {:>9}'
        # 打印每个实体的p, r, f
        for entity in self.entity_set:
            print(row_format.format(
                entity,
                self.precision_scores[entity],
                self.recall_scores[entity],
                self.f1_scores[entity],
                self.std_entity_counter[entity]   #这部分是support的值
            ))
        #计算并打印平均值
        avg_metrics = self._cal_wighted_average()
        print(row_format.format(
            'avg/total',
            avg_metrics['precision'],
            avg_metrics['recall'],
            avg_metrics['f1_score'],
            self.std_entity_number
        ))

    def _cal_wighted_average(self):
        #计算加权均值

        weighted_average = {}
        total = self.std_entity_number  #标准实体的总数

        #计算weighted precisions:
        weighted_average['precision'] = 0.
        weighted_average['recall'] = 0.
        weighted_average['f1_score'] = 0.
        for entity in self.entity_set:
            size = self.std_entity_counter[entity]  #标准结果各个实体的个数
            weighted_average['precision'] += self.precision_scores[entity] * size
            weighted_average['recall'] += self.recall_scores[entity] * size
            weighted_average['f1_score'] += self.f1_scores[entity] * size

        for metric in weighted_average.keys():
            weighted_average[metric] /= total

        return weighted_average

	#以下注释掉的代码为将B-company E-book这种BMSE格式符合情况但标签前后不一致，不计入预测结果的实体数中
    # def count_entity_dict(self, tag_list):
    #     """
    #     计算每个实体对应的个数，注意BME、BE、S结构才为实体，其余均不计入,B-company和E-book也不计入
    #     :param tag_list:
    #     :return:
    #     """
    #     enti_dict = {}
    #     flag = 0  # 初始状态设置为0
    #     B_word = ''  #初始状态B设为空
    #     for tag in tag_list:
    #         if 'B-' in tag and flag == 0:  #当B-出现时，将状态变为1
    #             flag = 1
    #             B_word = tag[2:]
    #         if 'M-' in tag and flag == 1:
    #             M_word = tag[2:]
    #             if M_word != B_word:   #当M和B标签不同时，不为实体将B_word设为空
    #                 B_word = ''
    #                 flag = 0
    #         if 'E-' in tag and flag == 1: #E前有B则可以判定为一个实体
    #             flag = 0  #状态回归初始
    #             E_word = tag[2:]
    #             tag = tag[2:]
    #             if E_word == B_word:
    #                 if tag not in enti_dict:
    #                     enti_dict[tag] = 1
    #                 else:
    #                     enti_dict[tag] += 1
    #             B_word = ''
    #         if 'S-' in tag:  #当S-出现，直接加一
    #             B_word = ''
    #             flag = 0
    #             tag = tag[2:]
    #             if tag not in enti_dict:
    #                 enti_dict[tag] = 1
    #             else:
    #                 enti_dict[tag] += 1
    #         if 'O' in tag: #出现O-时，将状态设为0 B_word设为0
    #             B_word = ''
    #             flag = 0
    #     return enti_dict

    def count_entity_dict(self, tag_list):
        """
        计算每个实体对应的个数，注意BME、BE、S结构才为实体，其余均不计入 B-company E-company算实体
        :param tag_list:
        :return:
        """
        enti_dict = {}
        flag = 0  # 初始状态设置为0
        for tag in tag_list:
            if 'B-' in tag and flag == 0:  #当B-出现时，将状态变为1
                flag = 1
            if 'E-' in tag and flag == 1: #E前有B则可以判定为一个实体
                flag = 0  #状态回归初始
                tag = tag[2:]
                if tag not in enti_dict:
                    enti_dict[tag] = 1
                else:
                    enti_dict[tag] += 1
            if 'S-' in tag:  #当S-出现，直接加一
                flag = 0
                tag = tag[2:]
                if tag not in enti_dict:
                    enti_dict[tag] = 1
                else:
                    enti_dict[tag] += 1
            if tag == 'O': #出现O-时，将状态设为0 B_word设为0
                flag = 0
        return enti_dict


    def count_correct_entity(self):
        """
        计算每种实体被正确预测的个数
        address、book、company、game、government、movie、name、organization、position、scene
        :return:
        """
        correct_enti_dict = {}
        flag = 0  #初始状态，表示等待下一个开始状态
        for std_tag, predict_tag in zip(self.std_tags, self.predict_tags):  #zip()用于将数据打包成元组
            if 'B-' in std_tag and std_tag == predict_tag and flag == 0:   #当以B-开头且标签相等时
                flag = 1  #表示已经有B
            if 'M-' in std_tag and std_tag == predict_tag and flag == 1:  #M前已经有过B
                flag = 1
            if 'E-' in std_tag and std_tag == predict_tag and flag == 1:
                flag = 0  #状态重新调整回初始值
                std_tag = std_tag[2:]
                if std_tag not in correct_enti_dict:
                    correct_enti_dict[std_tag] = 1
                else:
                    correct_enti_dict[std_tag] += 1
            if 'S-' in std_tag and std_tag == predict_tag and flag == 0:
                std_tag = std_tag[2:]
                if std_tag not in correct_enti_dict:
                    correct_enti_dict[std_tag] = 1
                else:
                    correct_enti_dict[std_tag] += 1
            if std_tag != predict_tag:
                flag = 0
        return correct_enti_dict


    def count_entity(self, tag_list):
        """
        计算标准列表中的实体个数，因为标准结果中无错误分类，所以实体的个数可以直接计算为E标签和S标签数目之和
        :return:
        """
        entity_count = 0   #记录实体数量
        for tag in tag_list:    #遍历所有标签
            if 'E-' in tag:
                entity_count += 1
            if 'S-' in tag:
                entity_count += 1
        return entity_count

def flatten_lists(lists):
    """
    将列表的列表拼成一个列表
    :param lists:
    :return:
    """
    flatten_list = []
    for l in lists:
        if type(l) == list:
            flatten_list += l
        else:
            flatten_list.append(l)
    return flatten_list

test_tag_lists = [["O", "O", "O", "B-S", "I-S", "O", ""]]
pred_tag_lists = [["O", "O", "O", "B-S", "I-S", "O", ""]]

metrics = Metrics(test_tag_lists, pred_tag_lists)
metrics.report_scores()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 14:56
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : io.py
from __future__ import annotations
import dataclasses
import json
from typing import Dict, Union, Text, Any, List
from pathlib import Path
import os

from liyi_cute.shared.exceptions import (FileIOException, FileNotFoundException)
from liyi_cute.shared.imports.schemas.schemaItem import ExampleSchema

from liyi_cute.shared.imports.schemas.schema import Example

DEFAULT_ENCODING = "utf-8"

def write_text_file(
    content: Text,
    file_path: Union[Text, Path],
    encoding: Text = DEFAULT_ENCODING,
    append: bool = False,
) -> None:
    """Writes text to a file. code come from
    Args:
        content: The content to write.
        file_path: The path to which the content should be written.
        encoding: The encoding which should be used.
        append: Whether to append to the file or to truncate the file.
    """
    mode = "a" if append else "w"
    with open(file_path, mode, encoding=encoding) as file:
        file.write(content)


def json_to_string(obj: Any, **kwargs: Any) -> Text:
    """Dumps a JSON-serializable object to string.
    code come from https://github.com/RasaHQ/rasa/blob/rasa/shared/nlu/training_data/formats/rasa.py#L109
    Args:
        obj: JSON-serializable object.
        kwargs: serialization options. Defaults to 2 space indentation
                and disable escaping of non-ASCII characters.
    Returns:
        The objects serialized to JSON, as a string.
    """
    indent = kwargs.pop("indent", 2)
    ensure_ascii = kwargs.pop("ensure_ascii", False)
    return json.dumps(obj, indent=indent, ensure_ascii=ensure_ascii, **kwargs)

def examples_as_json(examples: List[Example], **kwargs: Any)->str:
    """
    :param examples:
    :return:
    """
    examples =  ExampleSchema().dump(examples, many= True if isinstance(examples, List) else False)
    output = json_to_string(examples, indent=2, ensure_ascii=False, **kwargs)
    return output


def read_file(filename: Union[Text, Path], encoding: Text = DEFAULT_ENCODING) -> Any:
    """Read text from a file."""

    try:
        with open(filename, encoding=encoding) as f:
            return f.read()
    except FileNotFoundError:
        raise FileNotFoundException(
            f"Failed to read file, " f"'{os.path.abspath(filename)}' does not exist."
        )
    except UnicodeDecodeError:
        raise FileIOException(
            f"Failed to read file '{os.path.abspath(filename)}', "
            f"could not read the file using {encoding} to decode "
            f"it. Please make sure the file is stored with this "
            f"encoding.")


def read_json_file(filename: Union[Text, Path]) -> Any:
    """Read json from a file."""
    content = read_file(filename)
    try:
        return json.loads(content)
    except ValueError as e:
        raise FileIOException(f"Failed to read json from '{os.path.abspath(filename)}'. Error: {e}")


# -*- coding:UTF-8 -*-

# author:user
# contact: test@test.com
# datetime:2022/3/29 12:07
# software: PyCharm

"""
文件说明：
    
"""
from __future__ import annotations


from typing import Optional, Text, Union, Dict, List
from collections import Iterable
import re, os, logging

from tqdm import tqdm

from liyi_cute.shared.imports.base_parser import BasePaeser
from liyi_cute.shared.imports.schemas.schema import Example, Entity, Relation
from liyi_cute.utils.common import iter_file_groups

logger = logging.getLogger(__file__)

class BratParser(BasePaeser):
    """Parser for brat rapid annotation tool (Brat).

    Args:
        task_name: do work name
        ignore_types: types to ignore, should be in {"T", "R"}. (default: None)
        error : Error handling, should be in {"raise", "ignore"}.  (default: "raise")
    """
    exts = {".ann", ".txt"}
    support_save = {".pkl", ".json"}
    support_task_name = {"ner", "rel"} #  "ner", "rel" "env", "attr"
    types = {"T", "R"} # types = {"T", "R", "*", "E", "N", "AM"}
    def __init__(self, task_name, ignore_types: Optional[Iterable[str]] = None, error: str = "raise", lang="en"):
        super().__init__(task_name,ignore_types, error, lang)

    def parse(self, dirname: Union[str, bytes, os.PathLike], encoding: str = "utf-8") -> list[Example]:
        """Parse examples in given directory.

        Args:
            dirname : Directory  containing brat examples.
            task_name: work name
            encoding : Encoding for reading text files and ann files

        Returns:
            examples (list[Example]): Parsed examples.
        """
        examples = []
        file_groups = iter_file_groups(dirname, self.exts, missing="error" if self.error == "raise" else "ignore")
        for key, (ann_file, txt_file) in tqdm(file_groups):
            txt = self._parse_text(txt_file, encoding=encoding)
            ann = self._parse_ann(ann_file, encoding=encoding)
            examples.append(Example(task_name=self.task_name, id=key,text=txt, **ann))

        examples.sort(key=lambda x: x.id if x.id is not None else "")
        return examples


    def _parse_text(self, txt:Text, encoding:Text)->Text:  # pylint: disable=no-self-use
        with open(txt, mode="r", encoding=encoding) as f:
            return f.read()


    def _parse_ann(self, ann:Text, encoding:Text)->Dict:

        entity_matches, relation_matches = [], []
        with open(ann, mode="r", encoding=encoding) as f:
            for line in f:
                line = line.rstrip()

                if not line or line.startswith("#") or self._should_ignore_line(line):
                    continue
                ## 匹配实体
                if line.startswith("T"):
                    match = self._parse_entity(line)
                    if match:
                        entity_matches += [match]

                ## 匹配关系
                elif line.startswith("R"):
                    match = self._parse_relation(line)
                    if match:
                        relation_matches += [match]

                elif line.startswith("*"):
                    logger.warning(f"The '*' interface is not implemented, line: {line}")

                elif line.startswith("E"):
                    logger.warning(f"The 'E' interface is not implemented, line: {line}")

                elif line.startswith("N"):
                    logger.warning(f"The 'N' interface is not implemented, line: {line}")

                elif line.startswith("AM"):
                    logger.warning(f"The 'AM' interface is not implemented, line: {line}")

        # Format entities.
        entities = self._format_entities(entity_matches)
        self._check_entities(entities.values())
        if self.task_name=="ner":
            self.examples = {"entities": list(entities.values()), "relations": [], "events":[]}
            return self.examples

        # Format relations.
        relations = self._format_relations(relation_matches, entities)
        if self.task_name=="rel":
            self.examples = {"entities": list(entities.values()), "relations": relations, "events": []}
            return self.examples

        # Format events.
        # events = self._format_events(event_matches, entities) # "events": list(events.values()),
        self.examples = {"entities": list(entities.values()), "relations": [], "events":[]}
        return self.examples

    def _parse_entity(self, line):
        regex = re.compile(
            r"""(?P<id>T\d+)
                \t(?P<type>[^ ]+)
                \ (?P<start>\d+)
                \ (?P<end>\d+)
                \t(?P<mention>.+)""",
            re.X,
        )
        match = re.match(regex, line)
        if not match:
            self._raise_invalid_line_error(line)

        return match

    def _format_entities(self, entity_matches:Optional[List[re.compile]]):
        # pylint: disable=no-self-use
        return {
            match["id"]: Entity(
                mention=match["mention"],
                type=match["type"],
                start=int(match["start"]),
                end=int(match["end"]),
                id=match["id"],
            )
            for match in entity_matches
        }

    def _check_entities(self, entities:List):
        """pylint: disable=no-self-use"""
        pool = {}
        for entity in entities:
            id_ = pool.setdefault((entity.start, entity.end), entity.id)
            if id_ != entity.id:
                self._raise(RuntimeError("Detected identical span for"f" different entities: [{id_}, {entity.id}]"))


    def _parse_relation(self, line):
        regex = re.compile(
            r"""(?P<id>R\d+)
                \t(?P<type>[^ ]+)
                \ Arg[12]:(?P<arg1>T\d+)
                \ Arg[12]:(?P<arg2>T\d+)""",
            re.X,
        )
        match = re.match(regex, line)
        if not match:
            self._raise_invalid_line_error(line)

        return match

    def _format_relations(self, relation_matches, entities):
        relations = []
        for rel in relation_matches:
            arg1_id, arg2_id = rel["arg1"], rel["arg2"]
            arg1 = entities.get(arg1_id)
            arg2 = entities.get(arg2_id)
            if not arg1 or not arg2:
                self._raise(KeyError(f"Missing relation arg: {arg1_id if not arg1 else arg2_id}"))

            relations += [
                Relation(type=rel["type"], arg1=arg1, arg2=arg2, id=rel["id"])
            ]

        return relations




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# code come from https://github.com/RasaHQ/rasa/blob/rasa/shared/nlu/training_data/formats/readerwriter.py#L49
# @Time    : 2022/3/27 16:24
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : readerwriter.py
from __future__ import annotations

import abc
import json
from typing import Text, List, Any, Union, Dict
from pathlib import Path

from liyi_cute.utils.io import read_file, write_text_file
from liyi_cute.shared.imports.schemas.schema import Example

class ExamplesDataReader(abc.ABC):
    """Reader for NLU training data.
    """

    def __init__(self) -> None:
        """Creates reader instance."""
        self.filename: Text = ""

    def read(self, filename: Union[Text, Path], **kwargs: Any) -> "List[Example]":
        """Reads TrainingData from a file."""
        self.filename = str(filename)
        return self.reads(read_file(filename), **kwargs)

    @abc.abstractmethod
    def reads(self, s: Text, **kwargs: Any) -> "List[Example]":
        """Reads TrainingData from a string."""
        raise NotImplementedError

class ExamplseDataWriter:
    """A class for writing training data to a file."""

    def dump(self, filename: Text, example_data: "Example") -> None:
        """Writes a TrainingData object to a file."""
        s = self.dumps(example_data)
        write_text_file(s, filename)

    def dumps(self, example_data: "Example") -> Text:
        """Turns TrainingData into a string."""
        raise NotImplementedError

class JsonExamplesDataReader(ExamplesDataReader):
    """A class for reading JSON files."""

    def reads(self, s: Text, **kwargs: Any) -> "Example":
        """Transforms string into json object and passes it on."""
        js = json.loads(s)
        return self.read_from_json(js, **kwargs)

    def read_from_json(self, js: Dict[Text, Any], **kwargs: Any) -> "Example":
        """Reads TrainingData from a json object."""
        raise NotImplementedError


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 22:55
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : metric.py

"""
准确率: accuracy = 预测对的元素个数/总的元素个数
查准率：precision = 预测正确的实体个数 / 预测的实体总个数
召回率：recall = 预测正确的实体个数 / 标注的实体总个数
F1值：F1 = 2 *准确率 * 召回率 / (准确率 + 召回率)
"""
from __future__ import annotations

from typing import List
import time


from seqeval.metrics import f1_score
from seqeval.metrics import precision_score
from seqeval.metrics import accuracy_score
from seqeval.metrics import recall_score
from seqeval.metrics import classification_report


def nerMetrics(true_labels, pre_labels, logger):
    """
    :param true_labels 真实标签数据 [O,O,B-OR, I-OR]
    :param pre_labels 预测标签数据 [O,O,B-OR, I-OR]
    :param logger 日志实例
    """
    start_time = time.time()
    acc = accuracy_score(true_labels, pre_labels)
    f1score = f1_score(true_labels, pre_labels, average='macro')
    report = classification_report(true_labels, pre_labels, digits=4)
    msg = '\nTest Acc: {0:>6.2%}, Test f1: {1:>6.2%}'
    logger.info(msg.format(acc, f1score))
    logger.info("\nPrecision, Recall and F1-Score...")
    logger.info("\n{}".format(report))
    time_dif = time.time() - start_time
    logger.info("Time usage:{0:>.6}s".format(time_dif))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 12:13
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : jieba_tokenizer.py

from __future__ import annotations

from liyi_cute.processor.tokenizers.tokenizer import Tokenizer


class JiebaTokenizer(Tokenizer):
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 13:57
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : example_pmid_parse.py
import json
from liyi_cute.shared.imports.pmid_parser import PmidParser
from liyi_cute.shared.convert import convert_example_data
from liyi_cute.shared.imports.schemas.schemaItem import ExampleSchema

if __name__ == '__main__':
    obj = PmidParser(task_name="ner", error="ignore")
    examples = obj.parse(r"../data/pmid")
    a = obj.entities()
    mutation = list(a['Mutation'])
    gene = list(a["Gene"])
    chemical = list(a["Chemical"])

    with open("../tests/mutation.txt", "w", encoding="utf-8") as f:
        f.write("\n".join(mutation))

    with open("../tests/gene.txt", "w", encoding="utf-8") as f:
        f.write("\n".join(gene))

    with open("../tests/chemical.txt", "w", encoding="utf-8") as f:
        f.write("\n".join(chemical))


    # convert_example_data(examples, output_format="json", out_file=r"../data/output/pmid.json")

    # js = json.load(open("../data/output/pmid.json", "r", encoding='utf-8'))
    # ex = ExampleSchema()
    # print(ex.load(js, many=True))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/26 23:25
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : pmid.py
"""
解析pmid json文件内容
{"key":""
"text":"",
"annotations":[{},{}]
}
"""
from typing import List, Dict
from glob import iglob
import json
import os
import logging
from collections import OrderedDict

from tqdm import tqdm

logger = logging.getLogger(__file__)

def parse_pmid_json(path:str)->List:
    """
    :param path:
    :return:
    """
    pmid_files = iglob("{0}/*.json".format(path))
    informations = list()
    for index, file in tqdm(enumerate(pmid_files)):
        text, annotations = load_json(file)
        ## 核对标签文本是否对齐
        if not check_annotations(text, annotations):
            raise ValueError("label misaligned with text position, file path is{}".format(file))
        informations.append({"key":index, "path":file, "text":text, "annotations":annotations})

    return informations



def load_json(input_path):
    """
    :param input_path:
    :return:
    """
    data = json.load(open(input_path, 'r', encoding="utf-8"))
    text = ""
    annotations = []
    for item in data["passages"]:
        if item["text"]:
            text = text + item["text"] + " "
            for anns in item["annotations"]:
                if anns["text"].strip() != "":
                    annotations.append([anns["locations"][0]["offset"],
                                         anns["locations"][0]["offset"]+anns["locations"][0]["length"],
                                         anns["text"],
                                         anns["infons"]["type"]])
    text = text.rstrip()
    return text, annotations

def check_annotations(text:str, annotations:List) -> bool:
    """
    :param text:
    :param annotations:
    :return:
    """
    for an in annotations:
        start_id = an[0]
        end_ids = an[1]
        if text[start_id:end_ids] != an[2]:
            logger.warning("start_ids: " + str(start_id))
            logger.warning("end_ids: " + str(end_ids))
            logger.warning("text content: " + text[start_id:end_ids])
            logger.warning("annotations content: " + an[2])
            return False
    return True

def covert_json_to_ann(informations:Dict)->None:
    """
    :param informations: {"key":index,"text":text, "annotations":annotations}
    :return:None
    """
    if isinstance(informations, dict):
        raise ValueError("the type of annotations is not dict")
    if len(informations) and "key" not in informations[0].keys():
        raise ValueError("'key' not in dict")
    informations = sorted(informations, key=lambda x:x["key"])
    entity_index = 1
    for index, content in tqdm(enumerate(informations)):
        key = content['key']
        text = content['text']
        path = content['path']
        annotations = content['annotations']
        txt_file_w = open(os.path.splitext(path)[0]+".txt", "w", encoding="utf-8")
        ann_file_w = open(os.path.splitext(path)[0]+".ann", "w", encoding="utf-8")
        for ann in annotations:
            e_inx = "T"+str(entity_index)
            e_type = ann[3]
            strat_postion = ann[0]
            end_postion = ann[1]
            word = ann[2]
            ann_file_w.write(e_inx+"\t"+e_type+" "+str(strat_postion)+" "+str(end_postion)+"\t"+word)
            ann_file_w.write("\n")
            entity_index+=1
        txt_file_w.write(text)
        txt_file_w.close()
        ann_file_w.close()
    logger.info("files covert ann format fninsh")

def covert_ann_to_json():
    pass



if __name__ == '__main__':
    info = parse_pmid_json("../../datasets/pmid")
    covert_json_to_ann(info)
    print(1)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 16:18
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : example_tokenizer.py
from liyi_cute.processor.postprocess import NerPostprocessing
from liyi_cute.processor.preprocess import cut_sent
from liyi_cute.shared.imports.pmid_parser import PmidParser
from liyi_cute.processor.tokenizers.lm_tokenizer import LanguageModelTokenizer
from liyi_cute.shared.imports.schemas.schemaItem import TraningNerExampleSchema

obj = PmidParser(task_name="ner", error="ignore")
examples = obj.parse(r"../data/pmid")
#[{

# }]

cut_examples = cut_sent(examples, max_length=256)
obj = LanguageModelTokenizer.create({
        "pretrained_model_name_or_path": "bert-base-uncased",
        "add_special_tokens": True,
        "max_length": 256,
        "padding":"max_length",
        "return_offsets_mapping":True,
        "task_name":"ner"
    })
t = TraningNerExampleSchema(many=True)
tts = []
pre_tags = []
for c_exp in cut_examples:
    tts.append(obj.tokenize(c_exp))
    pre_tags.append(["O"]*len(c_exp.input_ids))

result = NerPostprocessing(pre_tags, tts)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/4 16:06
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : conll_parser.py
"""
conll 格式转换
Be  O
EU	B-ORG
rejects	O
German	B-MISC
call	O

序列化数据
Text: Be EU rejects German call
entities:[Entity(id = "T1", start=1, end=2, metion="EU"), type="MISC"]
"""

from __future__ import annotations

import os
from typing import Optional, Iterable, Union, Text, List, Tuple

from tqdm import tqdm

from liyi_cute.shared.exceptions import NotImplementedException
from liyi_cute.shared.imports.base_parser import BasePaeser
from liyi_cute.shared.imports.schemas.schema import Entity, Example
from liyi_cute.utils.common import bioes_to_bio, iter_file_groups
from liyi_cute.utils.io import read_file

class ConllParser(BasePaeser):
    exts = {".txt"}

    def __init__(self,task_name, ignore_types: Optional[Iterable[str]] = None, error: str = "raise"):
        super().__init__(task_name,ignore_types, error)

    def _load_data(self, file_path:Text, encoding:Text)-> Tuple[List[list], List[list]]:
        lines = read_file(file_path, encoding)

        line = lines.strip()
        all_data = []
        all_tag = []
        if line:
            lines_list = lines.split("\n\n")
            for line_s in lines_list:
                line_list = line_s.split("\n")
                data = []
                tag = []
                for line in line_list:
                    if not line.strip():
                        continue
                    data.append(line.split("\t")[0])
                    tag.append(line.split("\t")[1])
                # entity_obj = self._serialization_data(data, tag, mode)
                all_data.append(data)
                all_tag.append(tag)
        return all_data, all_tag

    def _parse_txt(self, file_path:Text, encoding:Text="utf-8", mode="train")->List[Example]:
        all_data, all_tag = self._load_data(file_path=file_path, encoding=encoding)
        ## 判断标注类型 BIO BIOES BMSE...
        tag_type = self._tag_type(all_tag)
        if tag_type == "BEIOS":
            new_all_tag = []
            for t in all_tag:
                new_all_tag.append(bioes_to_bio(t))
            all_tag = new_all_tag

        ## 序列化
        examples = []
        for idx in range(len(all_data)):
            examples.append(self._serialization_data(all_data[idx], all_tag[idx], mode))
        return examples


    def _en_serialization(self, data:List, tag:List, mode)->Example:
        text = ""
        entities = []
        tag_len = len(tag)
        index = 0
        ids = 1
        while index<tag_len:
            text = text + data[index] if index==0 else text + " "+ data[index]
            if tag[index][0]=="B":
                type = tag[index].split("-")[1]
                start = len(text) - len(data[index])
                mention = data[index]
                while index+1 <tag_len and tag[index+1][0]=="I":
                    index+=1
                    text += " " + data[index]
                    mention +=  " " + data[index]
                end = len(text)
                entities.append(Entity(id=f"T{str(ids)}", start=start, end=end, mention=mention.rstrip(), type=type))
                ids+=1
                index+=1
            else:
                index += 1
        return Example(id=mode, task_name=self.task_name, text=text, entities=entities)

    def _serialization_data(self, data:List, tag:List, mode:Text)->Example:
        if self.lang=="en":
            return self._en_serialization(data, tag, mode)
        elif self.lang=="ch":
            raise NotImplementedException
        else:
            raise NotImplementedException


    def parse(self, dirname: Union[str, bytes, os.PathLike], encoding: str = "utf-8"):
        examples = []
        file_groups = iter_file_groups(
            dirname,
            self.exts,
            missing="error" if self.error == "raise" else "ignore",
        )
        for key, file_path in tqdm(file_groups):
            examples+=self._parse_txt(file_path[0], encoding=encoding, mode=os.path.split(file_path[0])[1].split(".")[0])

        return examples




# -*- coding:UTF-8 -*-

# author:user
# contact: test@test.com
# datetime:2022/3/28 13:56
# software: PyCharm

"""
文件说明：
    序列化与反序列化
"""
from typing import Dict
# from pprint import pprint
# from liyi_cute.shared.imports.schemas.data_schema import Entity, EntitySchema
# en = Entity(mention="ssss", start=1, end=1, id="T1", type="gene")
# schema = EntitySchema()
# result = schema.dump(en)
# ## 序列化
# print(result)
# pprint(type(result))
# pprint(type(schema.dumps(en)))
#
# ## 过滤输出
# summary_schema = EntitySchema(only=("mention", "start"))
# rs = summary_schema.dump(en)
# pprint(rs)
#
# ## 反序列化
# res = schema.load(result)
# print(res)

## 对象之间的序列化与反序列话
# from liyi_cute.shared.imports.bart_parser import BratParser
# from liyi_cute.shared.imports.schemas.data_schema import ExampleSchema
# brat = BratParser(error="ignore")
# examples = brat.parse("../data/bio/")
# example_schema = ExampleSchema()
# exs = []
# # 序列化
# for ex in examples:
#     result = example_schema.dump(ex)
#     exs.append(result)
#
# ## 反序列化
# sez = []
# for ex in exs:
#     res = example_schema.load(ex)
#     sez.append(res)


## 从json文件中加载，然后反序列话
import json

from liyi_cute.shared.exceptions import NotContainException
from liyi_cute.shared.imports.schemas.data_schema import Example, Entity, Relation, Reference, Event

js = json.load(open("../data/output/test.json", 'r', encoding="utf-8"))
# a = JSONSerializer.deserialize(Example, js[0])

all_entity_dict = []
for example in js:
    text = example.get("text", "")
    if not text:
        continue
    entities = example.get("entities", [])
    ## 实体初始化

    entity_dict = {}
    for ent  in entities:
        if not ent.get("id"):
            raise ValueError
        references = ent.pop("references")
        references_list = [Reference(**data) for data in references ]
        entity_obj = Entity(references=references_list, **ent)

        entity_dict[ent.get("id")] = entity_obj
    all_entity_dict.append(entity_dict)

tasks = ["ner", "rel", "eve", "attr"] # 1)entity extraction 2)relation extraction 3)event extraction 4)attribute extraction
task_name = "ner"
if task_name not in tasks:
    raise NotContainException

for index, example in enumerate(js):
    text  = example.get("text", "")
    if not text:
        continue

    entitie_dict = all_entity_dict[index]
    relations = example.get("relations", [])
    events = example.get("events", [])
    id = example.get("id", None)





    # ## 关系初始化
    # relations = example.get("relations", [])
    # relation_list = []
    # for rel in relations:
    #     arg1 = rel.pop("arg1")
    #     arg1_references = arg1.pop("references")
    #     arg1_references_list = [Reference(**data) for data in arg1_references]
    #     arg1_obj = Entity(references = arg1_references_list, **arg1)
    #
    #     arg2 = rel.pop("arg2")
    #     arg2_references_list = arg2.pop("references")
    #     arg2_obj = Entity( references=arg2_references_list, **arg2)
    #
    #     rel_obj = Relation(arg1=arg1_obj, arg2=arg2_obj, **rel)
    #
    #     relation_list.append(rel_obj)
    #
    # ## 事件初始化
    # events = example.get("events", [])
    #
    # for env in events:
    #     trigger = env.pop("trigger")
    #     tri_references = trigger.pop("references")
    #     tri_reference_obj = [Reference(**data) for data in tri_references]
    #     trigger_obj = Entity(references=tri_reference_obj, **trigger)
    #
    #     argument_list = []
    #     arguments = env.pop("arguments")
    #     for arg_content in arguments:
    #         arg_content_references = arg_content.pop("references", [])
    #         arg_reference_obj = [Reference(**data) for data in arg_content_references]
    #         argument_list.append(Entity(references=arg_reference_obj, **arg_content))
    #
    #     event_obj = Event(trigger=trigger_obj, arguments=argument_list, **env)




def arguments_dfs(env:Dict):
    arguments = env.pop("arguments")
    if "arguments" in env:
        arguments_dfs(env)

    argument_list = []
    arguments = env.pop("arguments")
    for arg_content in arguments:
        arg_content_references = arg_content.pop("references", [])
        arg_reference_obj = [Reference(**data) for data in arg_content_references]
        argument_list.append(Entity(references=arg_reference_obj, **arg_content))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/5 15:18
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : setup.py.py

import setuptools

__version__ = None
exec(open('liyi_cute/version.py').read())

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


tests_requires = [
    "pytest"
]

install_requires = [
    "spacy>=2.3.0,<2.4.0",
    "marshmallow >= 3.15.0",
    "transformers >= 4.17.0",
    "setuptools-scm >= 6.4.2",
    "seqeval[cpu] == 1.2.2"
]

extras_requires = {
    'test': tests_requires
}

package_data = {
        "liyi_cute": ["*.py","*.so"]
    }

setuptools.setup(
    name="liyi-cute",
    version=__version__,
    author="yizheng dai",
    author_email="387942239@qq.com",
    maintainer="yizheng dai",
    maintainer_email="387942239@qq.com",
    license='Apache 2.0',
    description="A text processing tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/daiyizheng/liyi-cute",
    keywords="Text processing tools, including named entity recognition, "
                 "relation extraction, event extraction, and some statistical "
                 "and visualization functions",
    packages=setuptools.find_packages(),
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development",
    ],
    install_requires= install_requires,
    tests_require=tests_requires,
    python_requires='>=3.8',
    package_data=package_data,
    project_urls={
            'Bug Reports': 'https://github.com/daiyizheng/liyi-cute/issues',
            'Source': 'https://github.com/daiyizheng/liyi-cute',
        }
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/29 18:40
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : example_brat_parse.py.py
from liyi_cute.shared.exceptions import NotImplementedException
from liyi_cute.shared.imports.bart_parser import BratParser
from liyi_cute.shared.convert import convert_example_data
# Initialize a parser.
brat = BratParser(task_name="rel", error="ignore")
examples = brat.parse("../data/bio/")
convert_example_data(examples, output_format="json", out_file=r"../data/output/test.json")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 10:50
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : constants.py

from __future__ import annotations


GRADIENTS = ["linear-gradient(#FF2D2D, #ff7575, #FF9797)",
             "linear-gradient(#FF95CA, #ffaad5, #FFC1E0)",
             "linear-gradient(#FF8EFF, #ffa6ff, #FFBFFF)",
             "linear-gradient(#CA8EFF, #d3a4ff, #DCB5FF)",
             "linear-gradient(#AAAAFF, #B9B9FF, #CECEFF)",
             "linear-gradient(#84C1FF, #97CBFF, #ACD6FF)",
             "linear-gradient(#4DFFFF, #80FFFF, #A6FFFF)",
             "linear-gradient(#4EFEB3, #7AFEC6, #96FED1)",
             "linear-gradient(#79FF79, #93FF93, #A6FFA6)",
             "linear-gradient(#CCFF80, #D3FF93, #DEFFAC)",
             "linear-gradient(#FFFF93, #FFFFAA, #FFFFB9)",
             "linear-gradient(#FFE66F, #FFED97, #FFFOAC)",
             "linear-gradient(#FFC78E, #FFD1A4, #FFDCB9)",
             "linear-gradient(#FF9D6F, #FFAD86, #FFBD9D)",
             "linear-gradient(#C48888, #CF9E9E, #D9B3B3)",
             "linear-gradient(#B9B973, #C2C287, #CDCD9A)",
             "linear-gradient(#81C0C0, #95CACA, #A3D1D1)",
             "linear-gradient(#A6A6D2, #B8B8DC, #C7C7E2)",
             "linear-gradient(#C07AB8, #CA8EC2, #D2A2CC)",
             ]






































#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 17:39
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : liyi_example_data.py
from __future__ import annotations

from .readerwriter import ExamplesDataReader, ExamplseDataWriter, JsonExamplesDataReader

class LiyiWriter(ExamplesDataReader):
    pass
